'use strict';

const Gio = imports.gi.Gio;
const GObject = imports.gi.GObject;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;

const PopupMenu = imports.ui.popupMenu;
const MainLoop = imports.mainloop;
const Tweener = imports.ui.tweener;

const Config = imports.misc.config;
const SHELL_MINOR = parseInt(Config.PACKAGE_VERSION.split('.')[1]);

const Icon = {
    ON: 'icons/sand-clock-on.svg',
    OFF: 'icons/sand-clock-off.svg',
};


var Timer = {
    delayMinutes: 0,
    callback: function () { },
    timeoutID: 0,

    get countDownDate() {
        let milliseconds = this.delayMinutes * 60 * 1000;
        return new Date(new Date().getTime() + milliseconds);
    },

    start() {
        //  // Find the distance between now and the count down date
        let distance = this.countDownDate.setSeconds(0) - new Date();
        let delay = distance < 0 ? 0 : distance;

        Timer.clearTimeout();
        this.timeoutID = MainLoop.timeout_add(delay, () => {
            this.callback();
            this.reset();
            return false; // Stop repeating
        }, null);
    },

    reset() {
        this.clearTimeout();
        this.delayMinutes = 0;
    },

    clearTimeout() {
        if (this.timeoutID != 0) {
            MainLoop.source_remove(this.timeoutID);
            this.timeoutID = 0;
        }
    }
};

let notificationTextLabel;

var Indicator = class Indicator extends PanelMenu.Button {
    _init() {
        super._init(0.0, `${Me.metadata.name} Indicator`, false);

        this.icon = new St.Icon({ style_class: 'system-status-icon' });
        this.actor.add_child(this.icon);
        this._setPanelMenuIcon(Icon.OFF);

        this.timeLabel = new St.Label({ style_class: 'time-label' });

        // to prevent an empty label after sleep mode
        this._setTimeToLabel(new Date());

        this.messageEntry = new St.Entry({
            track_hover: false,
            can_focus: true,
            style_class: 'message-entry',
            text: "Knock knock"
        });

        let menuItem = new PopupMenu.PopupBaseMenuItem({ can_focus: false, reactive: false });
        menuItem.actor.add(this._makeUi());
        this.menu.addMenuItem(menuItem);

        this.menu.connect('open-state-changed', () => {
            if (Timer.delayMinutes == 0) {
                this._setTimeToLabel(new Date());
            }
        });

        Timer.callback = () => {
            this._setPanelMenuIcon(Icon.OFF);
            this._showMessage();
        }
    }

    _makeUi() {
        let oneBox = new St.BoxLayout({ vertical: false });
        oneBox.add(this._makeButtonsColon(['+10', '+15']));
        oneBox.add(this._makeButtonsColon(['+30', '+60']));

        let twoBox = new St.BoxLayout({ vertical: true });
        twoBox.add(this.timeLabel);
        twoBox.add(oneBox);

        let threeBox = new St.BoxLayout({ vertical: false });
        threeBox.add(twoBox);
        threeBox.add(this._makeButtonsColon(['0', '+1', '+5']));

        let mainBox = new St.BoxLayout({ vertical: true });
        mainBox.add(threeBox);
        mainBox.add(this.messageEntry);

        return mainBox;
    }

    _makeButtonsColon(labels) {
        let box = new St.BoxLayout({ vertical: true });
        labels.forEach(label => {
            box.add_child(this._createButton(label));
        });
        return box;
    }

    _setTimeToLabel(time) {
        this.timeLabel.text = time.toLocaleString('en-us', {
            hour12: false, hour: '2-digit', minute: '2-digit'
        });
    }

    _createButton(label) {
        let button = new St.Button({ label: label, style_class: 'number-button' });
        button.connect('clicked', () => {
            // reset timeout when pressed zero button
            if (button.label == '0') {
                Timer.reset();
            } else {
                Timer.delayMinutes += parseInt(button.label, 10);
            }

            Timer.start();

            this._setTimeToLabel(Timer.countDownDate);
            this._setPanelMenuIcon(Icon.ON);
        });

        return button;
    }

    _setPanelMenuIcon(icon) {
        this.icon.gicon = new Gio.FileIcon({
            file: Gio.File.new_for_path(Me.dir.get_child(icon).get_path())
        });
    }

    _showMessage() {
        if (!notificationTextLabel) {
            notificationTextLabel = new St.Label({ style_class: 'message-label' });
            Main.uiGroup.add_actor(notificationTextLabel);
        }

        notificationTextLabel.text = this.messageEntry.text;

        notificationTextLabel.opacity = 255;

        let monitor = Main.layoutManager.primaryMonitor;

        notificationTextLabel.set_position(monitor.x + Math.floor(monitor.width / 2 - notificationTextLabel.width / 2),
            monitor.y + Math.floor(monitor.height / 2 - notificationTextLabel.height / 2));

        Tweener.addTween(notificationTextLabel,
            {
                opacity: 0,
                time: 5,
                transition: 'easeInQuint',
                onComplete: hideMessage
            }
        );

        function hideMessage() {
            Main.uiGroup.remove_actor(notificationTextLabel);
            notificationTextLabel = null;
        }
    }
}

// Compatibility with gnome-shell >= 3.32
if (SHELL_MINOR > 30) {
    Indicator = GObject.registerClass(
        { GTypeName: 'Indicator' },
        Indicator
    );
}

// We're going to declare `indicator` in the scope of the whole script so it can
// be accessed in both `enable()` and `disable()`
var indicator = null;


function init() {
    log(`initializing ${Me.metadata.name} version ${Me.metadata.version}`);
}


function enable() {
    log(`enabling ${Me.metadata.name} version ${Me.metadata.version}`);

    indicator = new Indicator();

    // The `main` import is an example of file that is mostly live instances of
    // objects, rather than reusable code. `Main.panel` is the actual panel you
    // see at the top of the screen.
    Main.panel.addToStatusArea(`${Me.metadata.name} Indicator`, indicator);
}


function disable() {
    log(`disabling ${Me.metadata.name} version ${Me.metadata.version}`);

    // REMINDER: It's required for extensions to clean up after themselves when
    // they are disabled. This is required for approval during review!
    if (indicator !== null) {
        indicator.destroy();
        indicator = null;
    }
    Timer.reset();
}